#include <stdexcept>
#include <boost/test/auto_unit_test.hpp>

// Functions ----------------------------------------------------------------//

// Original declaration.
template<class output_type>
output_type do_something(int input_value)
throw (std::invalid_argument);

// This one works fine, because we declared the throw() clause properly.
template<>
int do_something(int input_value)
throw(std::invalid_argument)
{
	if (input_value < 0)
		throw std::invalid_argument("Input value too low.");
		
	return 1;
}

// This one does not work, because the throw() clause is empty.
template<>
long do_something(int input_value)
throw()
{
	if (input_value < 0)
		throw std::invalid_argument("Input value too low.");
		
	return 1;
}

// Test Cases ---------------------------------------------------------------//

BOOST_AUTO_TEST_SUITE(sigabrt_example_test_suite)

// Okay
BOOST_AUTO_TEST_CASE(int_no_throw)
{	
	BOOST_CHECK_EQUAL(do_something<int>(42), 1);
}

// Okay
BOOST_AUTO_TEST_CASE(int_throw)
{	
	BOOST_CHECK_THROW(do_something<int>(-1), std::invalid_argument);
}

// Okay
BOOST_AUTO_TEST_CASE(long_no_throw)
{	
	BOOST_CHECK_EQUAL(do_something<long>(42), 1);
}

// Fails, uncaught std::invalid_argument causes sigabrt
BOOST_AUTO_TEST_CASE(long_throw)
{	
	BOOST_CHECK_THROW(do_something<long>(-1), std::invalid_argument);
}

BOOST_AUTO_TEST_SUITE_END()
